require 'dspace'

class LearningObjectPublisher

  def initialize(dspace_client)
    @dspace = dspace_client
  end

  #create a learning object with draft status
  #if draft#type is 'external link', then the draft is published.
  def create_draft(draft)
    if draft.valid?
      item = create_dspace_item draft
      # save on postgres
      draft.id_dspace = item.id
      draft.save!

      publish! draft if draft.has_url_reference?
      return draft
    end
    false
  end

  #post *media_path* to *learning_object* on dspace
  def post(learning_object, media_path)
    DspaceUploadWorker.perform_async learning_object.id, learning_object.id_dspace, media_path, learning_object.description
  end

  ##publish *learning_object*
  #this method will create thumbnails, change state to published and will save it.
  def publish!(learning_object)
    if learning_object.is_draft?
      learning_object.publish
      ThumbnailGenerateWorker.perform_async learning_object.url_reference, learning_object.id if learning_object.has_url_reference?
    end
    learning_object.save
  end

  private

  def create_dspace_item(draft)
    item = @dspace.collections.create_item(build_dspace_item(draft), id: DspaceService::TEST_COLLECTION)

    # update dspace metadata
    metadata = ::Dspace::Metadata.new({'key' => 'dc.title', 'value' => draft.name, 'language' => draft.language.name})
    url_ref = ::Dspace::Metadata.new({'key' => 'dc.object.url', 'value' => draft.url_reference, 'language' => draft.language.name})
    @dspace.items.add_metadata(metadata, id: item.id)
    @dspace.items.add_metadata(url_ref, id: item.id) if draft.has_url_reference?
    item
  end

  def build_dspace_item(draft)
    ::Dspace::Item.new({
                           'name' => draft.name
                       })
  end

end
