require 'uri'
require 'screencap'

class ThumbnailGenerateWorker
  include Sidekiq::Worker

  def perform(media_path, attachment_id)
    media = create_media media_path
    attachment = ::LearningObject::Attachment.find(attachment_id)
    attachment.thumbnail = thumbnail_service.create_thumbnail media
    attachment.save!
  end

  private

  def create_media(path)
    return ::Screencap::Fetcher.new(path) if path =~ URI::regexp
    File.open path
  end

  def thumbnail_service
    @@thumbnailService ||= ThumbnailService.new(generators)
  end

  def generators
    [
        Thumbnail::Strategies::VideoThumbnailGenerator.new,
        Thumbnail::Strategies::ImageThumbnailGenerator.new,
        Thumbnail::Strategies::PdfThumbnailGenerator.new,
        Thumbnail::Strategies::UrlThumbnailGenerator.new
    ]
  end
end
